//+--------------------------------------------------------------------------------------------------------------------------------------------------------------------------------+
//|                                                                                                                                                                       Draw.mqh |
//|                                                                                                                                                                            hoz |
//|                                                                                                                                                                                |
//+--------------------------------------------------------------------------------------------------------------------------------------------------------------------------------+
#property copyright "hoz"
#property link      ""
#property version   "1.00"

// ======================================================================= Включения и импорт внешних модулей =====================================================================+
#include  <HOZ_Code\Enums\MessageType.mqh>

#import "Common\FileHandling.ex5"
  void      writeLog(ENUM_MESSAGE_TYPE messageType, string dataToWrite);
#import "EnvironmentInfo\TimeSeries.ex5"
  double    barHighPrice(int shift, ENUM_TIMEFRAMES tf = PERIOD_CURRENT);                             // Возвращает значение максимальной цены бара торгового инструмента активного окна, находящегося на shift баров от текущего бара в историю
  double    barHighPrice(int shift, string tradingInstrument, ENUM_TIMEFRAMES tf = PERIOD_CURRENT);   // Возвращает значение максимальной цены бара заданного торгового инструмента, находящегося на shift баров от текущего бара в историю
  double    barLowPrice(int shift, ENUM_TIMEFRAMES tf = PERIOD_CURRENT);                              // Возвращает значение минимальной цены бара торгового инструмента активного окна, находящегося на shift баров от текущего бара в историю
  double    barLowPrice(int shift, string tradingInstrument, ENUM_TIMEFRAMES tf = PERIOD_CURRENT);    // Возвращает значение минимальной цены бара заданного торгового инструмента, находящегося на shift баров от текущего бара в историю
  datetime  barOpenTime(int shift, ENUM_TIMEFRAMES tf = PERIOD_CURRENT);                              // Возвращает значение времени открытия бара торгового инструмента активного окна, находящегося на shift баров от 0-го бара в историю
  datetime  barOpenTime(int shift, string tradingInstrument, ENUM_TIMEFRAMES tf = PERIOD_CURRENT);    // Возвращает значение времени открытия бара заданного торгового инструмента, находящегося на shift баров от 0-го бара в историю
#import "Common\DataTypeRepresentation.ex5"
  string    iToS(const int valueToTransform);
#import

#include    <HOZ_Code\Enums\Location.mqh>
#include    <HOZ_Code\Classes\Environment\SymbolInstance.mqh>

#define WINGDINGS_NAME "WINGDINGS"    // Текстовое описание паттерна 'wingdings'
#define LINE_NAME "LINE"              // Текстовое описание паттерна 'line'
#define TEXT_NAME "TEXT"              // Текстовое описание паттерна 'text'
//+================================================================================================================================================================================+
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//|                                                                      Класс для отрисовки Wingdings-символов                                                                    |
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//+================================================================================================================================================================================+
class Draw {
  private:
    static double relativeDistance(const ENUM_TIMEFRAMES tf);
    static double relativeDistance(const double percent);
  public:
    //---- Конструктор и деструктор
    Draw(void);
   ~Draw(void);
    //---- Методы
    static string wingdings(string prefix, string objName, const ENUM_LOCATION objLocation, const long wingdingsCode, const int wingdingsWidth, const color wingdingsColor, const int barIndex, const ENUM_TIMEFRAMES tf);
    static string line(string prefix, string objName, const datetime time1, const double price1, const datetime time2, const double price2, const ENUM_LINE_STYLE lineStyle, const int lineWidth, const color lineColor, const int barIndex);
    static string text(string prefix, string objName, const ENUM_LOCATION objLocation, const double price, const color textColor, const int barIndex, const ENUM_TIMEFRAMES tf);
};
//==================================================================================================================================================================================
// Конструктор класса. =============================================================================================================================================================
Draw :: Draw(void) {
}
//==================================================================================================================================================================================
// Деструктор класса. ==============================================================================================================================================================
Draw :: ~Draw(void) {
}
//==================================================================================================================================================================================
// Возвращает расстояние равное запрошенному количеству процентов (percent) от экрана в зависимости от переданного таймфрейма. =====================================================
double Draw :: relativeDistance(const ENUM_TIMEFRAMES tf) {
  switch (tf) {
    case PERIOD_M1:
    case PERIOD_M5:
    case PERIOD_M15:
      return (ChartGetDouble(0, CHART_PRICE_MAX) - ChartGetDouble(0, CHART_PRICE_MIN)) / 100 * 4;
      break;
    case PERIOD_M30:
      return (ChartGetDouble(0, CHART_PRICE_MAX) - ChartGetDouble(0, CHART_PRICE_MIN)) / 100 * 4.5;
      break;
    case PERIOD_H1:
      return (ChartGetDouble(0, CHART_PRICE_MAX) - ChartGetDouble(0, CHART_PRICE_MIN)) / 100 * 3.5;
      break;
    case PERIOD_H4:
      return (ChartGetDouble(0, CHART_PRICE_MAX) - ChartGetDouble(0, CHART_PRICE_MIN)) / 100 * 4;
      break;
    case PERIOD_D1:
      return (ChartGetDouble(0, CHART_PRICE_MAX) - ChartGetDouble(0, CHART_PRICE_MIN)) / 100 * 4.5;
      break;
    case PERIOD_W1:
      return (ChartGetDouble(0, CHART_PRICE_MAX) - ChartGetDouble(0, CHART_PRICE_MIN)) / 100 * 4.5;
      break;
    case PERIOD_MN1:
      return (ChartGetDouble(0, CHART_PRICE_MAX) - ChartGetDouble(0, CHART_PRICE_MIN)) / 100 * 4.5;
      break;
    default:
      return (ChartGetDouble(0, CHART_PRICE_MAX) - ChartGetDouble(0, CHART_PRICE_MIN)) / 100 * 3;
  }
}
//==================================================================================================================================================================================
// Возвращает расстояние равное запрошенному количеству процентов (percent) от экрана. =============================================================================================
double Draw :: relativeDistance(const double percent) {
  return (ChartGetDouble(0, CHART_PRICE_MAX) - ChartGetDouble(0, CHART_PRICE_MIN)) / 100 * percent;
}
//==================================================================================================================================================================================
// Отрисовывает wingdings-символ над/под баром с индексом barIndex. ================================================================================================================
string Draw :: wingdings(string prefix,                     // Префикс
                         string objName,                    // Название объекта
                         const ENUM_LOCATION objLocation,   // Расположение wingdings-символа
                         const long wingdingsCode,          // Код wingdings-символа
                         const int wingdingsWidth,          // Ширина wingdings-символа
                         const color wingdingsColor,        // Цвет wingdings-символа
                         const int barIndex,                // Индекса бара, у которого будет отрисован wingdings-символ
                         const ENUM_TIMEFRAMES tf) {        // Таймфрем, на котором будет отрисован wingdings-символ
//---
  string objDescription = prefix + " " + WINGDINGS_NAME + " " + (string)barIndex + " " + TimeToString(barOpenTime(barIndex), TIME_DATE|TIME_MINUTES);

  if (!ObjectCreate(0, objDescription, OBJ_ARROW, 0, 0, 0)) {
    writeLog(MESSAGE_ERROR, __FUNCTION__ + " { Не удалось создать объект типа 'OBJ_ARROW' }");
    writeLog(MESSAGE_ERROR, __FUNCTION__ + " { _LastError = '" + iToS(_LastError) + "' }");
  }
  ObjectSetInteger(0, objDescription, OBJPROP_TIME, 0, barOpenTime(barIndex));
//  Print(ObjectGetDouble(0, objDescription, OBJPROP_PRICE));

  if (objLocation == LOCATION_ABOVE) {
    ObjectSetInteger(0, objDescription, OBJPROP_ANCHOR, ANCHOR_TOP);
    ObjectSetDouble(0, objDescription, OBJPROP_PRICE, barHighPrice(barIndex) + relativeDistance(tf));
  }
  else if (objLocation == LOCATION_BELOW) {
    ObjectSetInteger(0, objDescription, OBJPROP_ANCHOR, ANCHOR_BOTTOM);
    ObjectSetDouble(0, objDescription, OBJPROP_PRICE, barLowPrice(barIndex) - relativeDistance(tf));
  }
  ObjectSetInteger(0, objDescription, OBJPROP_ARROWCODE, wingdingsCode);
  ObjectSetInteger(0, objDescription, OBJPROP_WIDTH, wingdingsWidth);
  ObjectSetInteger(0, objDescription, OBJPROP_COLOR, wingdingsColor);

  ChartRedraw();
//---
  return objDescription;
}
//==================================================================================================================================================================================
// Отрисовывает линию по двум точкам. ==============================================================================================================================================
string Draw :: line(string prefix,                      // Префикс
                    string objName,                     // Название объекта
                    const datetime time1,               // Время 1-ой точки
                    const double price1,                // Цена 1-ой точки
                    const datetime time2,               // Время 2-ой точки
                    const double price2,                // Цена 2-ой точки
                    const ENUM_LINE_STYLE lineStyle,    // Стиль лишнии
                    const int lineWidth,                // Ширина трендовой линии
                    const color lineColor,              // Цвет трендовой линии
                    const int barIndex) {               // Индекса бара, у которого будет отрисован wingdings-символ
//---
  string objDescription = prefix + " " + LINE_NAME + " " + (string)barIndex + " " + TimeToString(time1, TIME_DATE|TIME_MINUTES) + " : " + TimeToString(time2, TIME_DATE|TIME_MINUTES);

  if (!ObjectCreate(0, objDescription, OBJ_TREND, 0, time1, price1, time2, price2)) {
    writeLog(MESSAGE_ERROR, __FUNCTION__ + " { Не удалось создать объект типа 'OBJ_TREND' }");
    writeLog(MESSAGE_ERROR, __FUNCTION__ + " { _LastError = '" + iToS(_LastError) + "' }");
  }
  ObjectSetInteger(0, objDescription, OBJPROP_STYLE, lineStyle);
  ObjectSetInteger(0, objDescription, OBJPROP_WIDTH, lineWidth);
  ObjectSetInteger(0, objDescription, OBJPROP_COLOR, lineColor);

  ChartRedraw();
//---
  return objDescription;
}
//==================================================================================================================================================================================
// Отображает текст над/под баром с индексом barIndex. =============================================================================================================================
string Draw :: text(string prefix,                      // Префикс
                    string objName,                     // Название объекта
                    const ENUM_LOCATION objLocation,    // Расположение текста
                    const double price,                 // Цена, над / под которой будет напечатан текст
                    const color textColor,              // Цвет текста
                    const int barIndex,                 // Индекса бара, у которого будет напечатан текст
                    const ENUM_TIMEFRAMES tf) {         // Таймфрем, на котором будет напечатан текст
//---
  string objDescription = prefix + " " + objName + " " + TEXT_NAME + " " + (string)barIndex + " " + TimeToString(barOpenTime(barIndex), TIME_DATE|TIME_MINUTES);

  if (!ObjectCreate(0, objDescription, OBJ_TEXT, 0, 0, 0)) {
    writeLog(MESSAGE_ERROR, __FUNCTION__ + " { Не удалось создать объект типа 'OBJ_TEXT' }");
    writeLog(MESSAGE_ERROR, __FUNCTION__ + " { _LastError = '" + iToS(_LastError) + "' }");
  }
  ObjectSetInteger(0, objDescription, OBJPROP_TIME, 0, barOpenTime(barIndex));

  if (objLocation == LOCATION_ABOVE) {
    ObjectSetInteger(0, objDescription, OBJPROP_ANCHOR, ANCHOR_TOP);
    ObjectSetDouble(0, objDescription, OBJPROP_PRICE, barHighPrice(barIndex) + relativeDistance(tf) * 1.6);
  }
  else if (objLocation == LOCATION_BELOW) {
    ObjectSetInteger(0, objDescription, OBJPROP_ANCHOR, ANCHOR_BOTTOM);
    ObjectSetDouble(0, objDescription, OBJPROP_PRICE, barLowPrice(barIndex) - relativeDistance(tf) * 1.5);
  }

  ObjectSetString(0, objDescription, OBJPROP_NAME, objDescription);
  ObjectSetString(0, objDescription, OBJPROP_TEXT, objName);
  ObjectSetInteger(0, objDescription, OBJPROP_FONTSIZE, 10);
  ObjectSetInteger(0, objDescription, OBJPROP_COLOR, textColor);

  ChartRedraw();
//---
  return objDescription;
}
